==> bitfront-library-6.3.0.tgz: package/src/app/base/component/store/base-sublist.component.ts <==
import { EventEmitter, Input, Output, SimpleChange, ViewChild } from "@angular/core";
import { Store } from "@ngrx/store";
import { Observable } from "rxjs";

import { Item, FormType, MessageService } from "../../../shared";
import { BitToolbarComponent } from "../../../shared/component/bit-toolbar.component";

import * as fromModels from "../../models";

/**
 * Componente que implementa un sub listado
 * T: entidad,
 * V: Definicion de la interface del state
 */
export class BaseSublistComponent<T, V> {
  displayInnerForm$: Observable<boolean>; // si tenemos que mostrar el form para editar el objeto en linea o no
  modeInnerForm$: Observable<FormType>; // modo en que mostramos el inner form
  itemSelected$: Observable<T>; // objeto a enviar al InnerForm
  selectedItems$: Observable<number[]>; // elementos checkeados
  items$: Observable<T[]>;
  toolbar$: Observable<{}>;

  @ViewChild("tableview")
  tableView: any;
  @Input()
  parent: any; // objeto padre que es dueño de la lista
  @Input()
  parentField: string; // propiedad de los elementos que establece la relación con su padre
  @Input()
  initLoad: boolean; // cuando sea true la lista se cargará
  @Input()
  itemsPreloaded: Item[]; // lista precargada de elementos que el padre nos inyecta
  @Input()
  femenino: boolean = false; // si tenemos que mostrar los textos en femenino o no (p. ej., Nou vs Nova)
  @Input()
  forceReload: EventEmitter<any>; // opcional, eventos a esta cola nos dicen que debemos forzar una recarga
  @Input()
  reloadFromServer: boolean = true; //indica si una vez añadido un elemento, llamaos al servidor para recargar la lista. Por defecto sí lo hace.
  @Output()
  private onChange = new EventEmitter<boolean>();

  constructor(
    protected store: Store<V>,
    protected messageService: MessageService,
    protected factoryCreateAction: fromModels.FactoryCreateAction,
    protected fnCreateNewObject?: fromModels.FactoryCreateNewObject<T>,
    protected fnCopyObject?: fromModels.FactoryCopyObject<T>
  ) {}

  ngOnInit() {
    this.loadItems();
  }

  /** Controlamos si la propiedad initLoad cambia a true para cargar la lista de documentos */
  ngOnChanges(changes: { [propKey: string]: SimpleChange }) {
    // console.log('BaseSublist OnChanges', changes);
    for (let propName in changes) {
      if (propName === "forceReload") {
        this.forceReload &&
          this.forceReload.subscribe(
            function(x) {
              this.loadItems();
            }.bind(this)
          );
      }
    }
  }

  /** Mostramos el inner form. */
  gotoForm(data?: any) {
    console.log("showForm called");
    if (data) {
      this.store.dispatch(
        this.factoryCreateAction(fromModels.GENERIC_TYPE.SHOW_ITEM_FOR_UPDATE, this.fnCopyObject(data))
      );
    } else {
      this.store.dispatch(
        this.factoryCreateAction(fromModels.GENERIC_TYPE.SHOW_ITEM_FOR_INSERT, this.fnCreateNewObject(this.parent.id))
      );
    }
  } /** metodos de BaseSubListComponent */

  //Nos presionan algún botón del toolbar
  onToolbarButtonPressed(button: string) {
    console.log(`Button pressed: ${button}`);
    if (button === BitToolbarComponent.NEW_BUTTON) {
      this.gotoForm();
    } else if (button === BitToolbarComponent.DELETE_BUTTON) {
      let p = this.messageService.confirm(
        "Està segur que vol eliminar els elements seleccionats? Aquesta acciò no es podrà desfer."
      );
      let local = this;
      p.then(function(confirmacion) {
        if (confirmacion) {
          local.deleteItems();
        }
      });
    } else {
      this.otherAction(button);
    }
  }

  /** Cualquier otra accion que no sea la típica de guardar o eliminar */
  otherAction(action: string): void {
    console.warn(`otherAction ${action} pressed`);
  }

  deleteItem(id: number): void {
    console.log(`deleteItem called for id ${id}`);
    this.store.dispatch(this.factoryCreateAction(fromModels.GENERIC_TYPE.DELETE_ITEMS, [id]));
    //this.onChange.emit(true); <----tenemos que emitir este valor cuando se haya eliminado el elemento
  }

  /** Inserta o actualiza el documento */
  saveItem(item: T): void {
    console.log(`saveItem called for item ${item}`);
    this.store.dispatch(this.factoryCreateAction(fromModels.GENERIC_TYPE.SAVE_ITEM, item));
  }

  loadItems(): void {
    console.log(`load items called for campanya id ${this.parent.id}`);
    this.store.dispatch(this.factoryCreateAction(fromModels.GENERIC_TYPE.LOAD_ITEMS, this.parent));
  }

  /** nos piden cerrar el formulario */
  closeForm(event) {
    this.store.dispatch(this.factoryCreateAction(fromModels.GENERIC_TYPE.CLOSE_ITEM));
  }

  /** Nos checkean algún item del listado */
  onItemChecked(selectedItems: number[]) {
    console.log("element selected");
    this.store.dispatch(this.factoryCreateAction(fromModels.GENERIC_TYPE.ITEMS_SELECTED, selectedItems));
    //TODO Implementado pero no verificado
    /*
        this.selectedItems = selectedItems;
        if (this.toolbar[BitToolbarComponent.DELETE_BUTTON]) {
          if (selectedItems !== null && selectedItems.length > 0) {
            this.toolbar[BitToolbarComponent.DELETE_BUTTON].visible = true;
          } else {
            this.toolbar[BitToolbarComponent.DELETE_BUTTON].visible = false;
          }
          this.updateToolbar();
        }*/
  }

  /** Elimina los documentos seleccionados */
  deleteItems(): void {
    console.log(
      "delete items"
    ); /*
    this.store //Ojo que al borrarse los enlaces la propiedad itemsSelected se pone a [] y esta accion podría volver a dispararse
      .select(fromStore.getCampanyaEnlacesItemsSelected)
      .subscribe(enlaces => {
        if (enlaces && enlaces.length > 0) {
          this.store.dispatch(new fromStore.DeleteEnlaces(enlaces));
        } else {
          if (this.tableView != null) this.tableView.selectedItems = [];
        }
      });
      */
    //TODO Implementado pero no verificado
    /*
      this.callDeleteItems(this.parent, items).subscribe(result => {
        this.lista = this.lista.filter(item => items.indexOf("" + item.id) < 0);
        this.postDelete(result);
      });
      */
  }

  /** Función para hacer algo después de haber eliminado los elementos seleccionados */
  postDelete(result: any) {
    console.log("postDelete llamado");
    //TODO Implementado pero no verificado
    /*
      this.toolbar[BitToolbarComponent.DELETE_BUTTON].visible = false;
      this.updateToolbar();
      if (this.tableView != null) this.tableView.selectedItems = []; <----tenemos que actualizar este valor cuando se haya eliminado el elemento
      */
  }
}

==> bitfront-library-6.6.2.tgz: package/src/app/base/component/store/base-sublist.component.ts <==
import { EventEmitter, Input, Output, SimpleChange, ViewChild } from "@angular/core";
import { Store } from "@ngrx/store";
import { Observable } from "rxjs";

import { Item, FormType, MessageService } from "../../../shared";
import { BitToolbarComponent } from "../../../shared/component/bit-toolbar.component";

import * as fromModels from "../../models";

import { BaseComponent } from "./base.component";

/**
 * Componente que implementa un sub listado
 * T: entidad,
 * V: Definicion de la interface del state
 */
export class BaseSublistComponent<T, V> extends BaseComponent<V> {
  displayInnerForm$: Observable<boolean>; // si tenemos que mostrar el form para editar el objeto en linea o no
  modeInnerForm$: Observable<FormType>; // modo en que mostramos el inner form
  itemSelected$: Observable<T>; // objeto a enviar al InnerForm
  selectedItems$: Observable<number[]>; // elementos checkeados
  items$: Observable<T[]>;
  toolbar$: Observable<{}>;

  @ViewChild("tableview")
  tableView: any;
  @Input()
  parent: any; // objeto padre que es dueño de la lista
  @Input()
  parentField: string; // propiedad de los elementos que establece la relación con su padre
  @Input()
  initLoad: boolean; // cuando sea true la lista se cargará
  @Input()
  itemsPreloaded: Item[]; // lista precargada de elementos que el padre nos inyecta
  @Input()
  femenino: boolean = false; // si tenemos que mostrar los textos en femenino o no (p. ej., Nou vs Nova)
  @Input()
  forceReload: EventEmitter<any>; // opcional, eventos a esta cola nos dicen que debemos forzar una recarga
  @Input()
  reloadFromServer: boolean = true; //indica si una vez añadido un elemento, llamaos al servidor para recargar la lista. Por defecto sí lo hace.
  @Output()
  private onChange = new EventEmitter<boolean>();

  constructor(
    protected store: Store<V>,
    protected messageService: MessageService,
    protected factoryCreateAction: fromModels.FactoryCreateAction,
    protected fnCreateNewObject?: fromModels.FactoryCreateNewObject<T>,
    protected fnCopyObject?: fromModels.FactoryCopyObject<T>
  ) {
    super(store);
  }

  ngOnInit() {
    this.loadItems();
  }

  ngOnDestroy() {
    super.ngOnDestroy();
  }

  /** Controlamos si la propiedad initLoad cambia a true para cargar la lista de documentos */
  ngOnChanges(changes: { [propKey: string]: SimpleChange }) {
    // console.log('BaseSublist OnChanges', changes);
    for (let propName in changes) {
      if (propName === "forceReload") {
        this.forceReload &&
          this.forceReload.subscribe(
            function(x) {
              this.loadItems();
            }.bind(this)
          );
      }
    }
  }

  /** Mostramos el inner form. */
  gotoForm(data?: any) {
    console.log("showForm called");
    if (data) {
      this.store.dispatch(
        this.factoryCreateAction(fromModels.GENERIC_TYPE.SHOW_ITEM_FOR_UPDATE, this.fnCopyObject(data))
      );
    } else {
      this.store.dispatch(
        this.factoryCreateAction(fromModels.GENERIC_TYPE.SHOW_ITEM_FOR_INSERT, this.fnCreateNewObject(this.parent.id))
      );
    }
  } /** metodos de BaseSubListComponent */

  //Nos presionan algún botón del toolbar
  onToolbarButtonPressed(button: string) {
    console.log(`Button pressed: ${button}`);
    if (button === BitToolbarComponent.NEW_BUTTON) {
      this.gotoForm();
    } else if (button === BitToolbarComponent.DELETE_BUTTON) {
      let p = this.messageService.confirm(
        "Està segur que vol eliminar els elements seleccionats? Aquesta acciò no es podrà desfer."
      );
      let local = this;
      p.then(function(confirmacion) {
        if (confirmacion) {
          local.deleteItems();
        }
      });
    } else {
      this.otherAction(button);
    }
  }

  /** Cualquier otra accion que no sea la típica de guardar o eliminar */
  otherAction(action: string): void {
    console.warn(`otherAction ${action} pressed`);
  }

  deleteItem(id: number): void {
    console.log(`deleteItem called for id ${id}`);
    this.store.dispatch(this.factoryCreateAction(fromModels.GENERIC_TYPE.DELETE_ITEMS, [id]));
    //this.onChange.emit(true); <----tenemos que emitir este valor cuando se haya eliminado el elemento
  }

  /** Inserta o actualiza el documento */
  saveItem(item: T): void {
    console.log(`saveItem called for item ${item}`);
    this.store.dispatch(this.factoryCreateAction(fromModels.GENERIC_TYPE.SAVE_ITEM, item));
  }

  loadItems(): void {
    console.log(`load items called for campanya id ${this.parent.id}`);
    this.store.dispatch(this.factoryCreateAction(fromModels.GENERIC_TYPE.LOAD_ITEMS, this.parent));
  }

  /** nos piden cerrar el formulario */
  closeForm(event) {
    this.store.dispatch(this.factoryCreateAction(fromModels.GENERIC_TYPE.CLOSE_ITEM));
  }

  /** Nos checkean algún item del listado */
  onItemChecked(selectedItems: number[]) {
    console.log("element selected");
    this.store.dispatch(this.factoryCreateAction(fromModels.GENERIC_TYPE.ITEMS_SELECTED, selectedItems));
    //TODO Implementado pero no verificado
    /*
        this.selectedItems = selectedItems;
        if (this.toolbar[BitToolbarComponent.DELETE_BUTTON]) {
          if (selectedItems !== null && selectedItems.length > 0) {
            this.toolbar[BitToolbarComponent.DELETE_BUTTON].visible = true;
          } else {
            this.toolbar[BitToolbarComponent.DELETE_BUTTON].visible = false;
          }
          this.updateToolbar();
        }*/
  }

  /** Elimina los documentos seleccionados */
  deleteItems(): void {
    console.log(
      "delete items"
    ); /*
    this.store //Ojo que al borrarse los enlaces la propiedad itemsSelected se pone a [] y esta accion podría volver a dispararse
      .select(fromStore.getCampanyaEnlacesItemsSelected)
      .subscribe(enlaces => {
        if (enlaces && enlaces.length > 0) {
          this.store.dispatch(new fromStore.DeleteEnlaces(enlaces));
        } else {
          if (this.tableView != null) this.tableView.selectedItems = [];
        }
      });
      */
    //TODO Implementado pero no verificado
    /*
      this.callDeleteItems(this.parent, items).subscribe(result => {
        this.lista = this.lista.filter(item => items.indexOf("" + item.id) < 0);
        this.postDelete(result);
      });
      */
  }

  /** Función para hacer algo después de haber eliminado los elementos seleccionados */
  postDelete(result: any) {
    console.log("postDelete llamado");
    //TODO Implementado pero no verificado
    /*
      this.toolbar[BitToolbarComponent.DELETE_BUTTON].visible = false;
      this.updateToolbar();
      if (this.tableView != null) this.tableView.selectedItems = []; <----tenemos que actualizar este valor cuando se haya eliminado el elemento
      */
  }
}
